@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.h,v 1.24 2004/01/18 02:26:28 fuyu Exp $
+ * $Id: bbs_thread_ui.h,v 1.24.2.1 2004/05/04 11:44:51 fuyu Exp $
  */
 
 #ifndef _BBSTHREAD_UI_H_
@@ -79,6 +79,7 @@ typedef struct _BBSThreadGUIInfo
    * last_read/next_markϤviewФΡ
    */
   GtkWidget *recent_view;
+  IconLabel *recent_tab_label;
 
   /* ǽο쥹Ƭդޡ */
   GtkTextMark *last_read;
@@ -104,6 +105,11 @@ typedef struct _BBSThreadGUIInfo
 
   gboolean rendering_done;
 
+  int hisshi_count;
+  guint auto_refresher_id;
+  GtkWidget *hisshidana_dialog;
+
+
   /* ɽΥΥե(DATե)ΥХåե */
   OchushaAsyncBuffer *response_source_buffer;
 
@@ -145,7 +151,8 @@ GtkWidget *open_bbs_thread(OchushaApplication *application,
 			   OchushaBBSThread *thread, IconLabel *tab_label);
 
 void refresh_thread(OchushaApplication *application, GtkWidget *widget,
-		    OchushaBBSThread *thread, IconLabel *tab_label);
+		    OchushaBBSThread *thread, IconLabel *tab_label,
+		    gboolean from_timeout);
 
 void save_thread_view_position(GtkWidget *view, OchushaBBSThread *thread);
 
