@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.86 2004/02/28 22:57:33 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.92.2.2 2004/05/04 09:16:20 fuyu Exp $
  */
 
 #include "config.h"
@@ -115,11 +115,11 @@ static void boardlist_read_board_element_cb(OchushaBBSTable *table,
 
 static void boardlist_write_category_element_cb(OchushaBBSTable *table,
 						OchushaBoardCategory *category,
-						gzFile *boardlist_xml,
+						gzFile boardlist_xml,
 						OchushaApplication *app);
 static void boardlist_write_board_element_cb(OchushaBBSTable *table,
 					     OchushaBulletinBoard *board,
-					     gzFile *boardlist_xml,
+					     gzFile boardlist_xml,
 					     OchushaApplication *app);
 
 static void write_threadlist(gpointer data, gpointer user_data);
@@ -128,7 +128,7 @@ static void build_boardlist(WorkerThread *employee, gpointer args);
 
 static void setup_boardlist(OchushaApplication *application);
 
-static void filter_toggled_cb(GtkToggleButton *toggle_button,
+static void filter_toggled_cb(GObject *unused,
 			      OchushaApplication *application);
 static void item_view_required_cb(PanedNotebook *paned_notebook,
 				  gpointer item, GtkWidget **item_view,
@@ -144,14 +144,18 @@ static void item_view_closed_cb(PanedNotebook *paned_notebook,
 static void page_switched_cb(PanedNotebook *paned_notebook,
 			     GtkWidget *previous_page, GtkWidget *new_page,
 			     OchushaApplication *application);
+static void page_double_clicked_cb(PanedNotebook *paned_notebook,
+				   OchushaBulletinBoard *board,
+				   GtkWidget *item_view,
+				   OchushaApplication *application);
 
-static void make_new_thread_button_cb(GtkWidget *widget,
+static void make_new_thread_button_cb(GObject *unused,
 				      OchushaApplication *application);
-static void open_board_properties_button_cb(GtkWidget *widget,
+static void open_board_properties_button_cb(GObject *unused,
 					    OchushaApplication *application);
-static void refresh_board_button_cb(GtkWidget *widget,
+static void refresh_board_button_cb(GObject *unused,
 				    OchushaApplication *application);
-static void start_threadlist_search_button_cb(GtkWidget *widget,
+static void start_threadlist_search_button_cb(GObject *unused,
 					      OchushaApplication *application);
 static void new_thread_dialog_response_cb(GtkWidget *dialog, int response_id,
 					  OchushaApplication *application);
@@ -304,9 +308,62 @@ ensure_my_folder(OchushaApplication *application)
 void
 initialize_boardlist(OchushaApplication *application)
 {
-  GtkToolbar *toolbar;
   GtkWidget *scrolled_window;
   GtkWidget *filter_button;
+#if GTK_MINOR_VERSION <= 2
+  GtkToolbar *toolbar;
+#else
+  GtkActionGroup *action_group;
+  GtkUIManager *ui_manager;
+  static GtkActionEntry toolbar_entries[] =
+    {
+      {
+	"CreateNewThread", GTK_STOCK_NEW, NULL, NULL,
+	N_("Make a New Thread at the Current Board"),
+	(GCallback)make_new_thread_button_cb
+      },
+      {
+	"OpenBoardProperties", GTK_STOCK_PROPERTIES, NULL, NULL,
+	N_("Open the Current Board's Properties"),
+	(GCallback)open_board_properties_button_cb
+      },
+      {
+	"SearchOnThreadlist", GTK_STOCK_FIND, NULL, NULL,
+	N_("Start Text Search on the Current Threadlist"),
+	(GCallback)start_threadlist_search_button_cb
+      },
+      {
+	"RefreshThreadlist", GTK_STOCK_REFRESH, NULL, NULL,
+	N_("Refresh the Current Threadlist"),
+	(GCallback)refresh_board_button_cb
+      }
+    };
+
+  static GtkToggleActionEntry toolbar_toggle_entries[] =
+    {
+      {
+	"ApplyFilter", NULL, N_("Filter"), NULL,
+	N_("Apply Filter to the Current Threadlist"),
+	(GCallback)filter_toggled_cb,
+	FALSE
+      }
+    };
+
+  static const char *toolbar_ui =
+    "<ui>"
+    "  <toolbar name=\"PanedNotebookLeftToolbar\">"
+    "    <toolitem name=\"Refresh\" action=\"RefreshThreadlist\"/>"
+    "    <toolitem name=\"Search\" action=\"SearchOnThreadlist\"/>"
+    "    <placeholder name=\"BoardToolItems\">"
+    "      <toolitem name=\"Filter\" action=\"ApplyFilter\"/>"
+    "      <toolitem name=\"Properties\" action=\"OpenBoardProperties\"/>"
+    "    </placeholder>"
+    "    <toolitem name=\"NewThread\" action=\"CreateNewThread\"/>"
+    "  </toolbar>"
+    "</ui>";
+
+  GError *error;
+#endif
 
   board_list = NULL;
 
@@ -376,6 +433,7 @@ initialize_boardlist(OchushaApplication *application)
 				application->board_tab_always_show
 				? GTK_POLICY_ALWAYS : GTK_POLICY_AUTOMATIC);
 
+#if GTK_MINOR_VERSION <= 2
   toolbar = paned_notebook_get_toolbar(PANED_NOTEBOOK(application->contents_window));
 
   gtk_toolbar_prepend_space(toolbar);
@@ -402,11 +460,11 @@ initialize_boardlist(OchushaApplication *application)
 			     "apply_filter_to_the_current_threadlist");
   g_signal_connect(G_OBJECT(filter_button), "toggled",
 		   G_CALLBACK(filter_toggled_cb), application);
-#if TRACE_MEMORY_USAGE
+# if TRACE_MEMORY_USAGE
   g_signal_connect(G_OBJECT(filter_button), "destroy",
 		   G_CALLBACK(gtk_widget_destroyed),
 		   &application->filter_button);
-#endif
+# endif
 
   gtk_toolbar_prepend_space(toolbar);
   gtk_toolbar_insert_stock(toolbar, GTK_STOCK_FIND,
@@ -423,17 +481,51 @@ initialize_boardlist(OchushaApplication *application)
 			   GTK_SIGNAL_FUNC(refresh_board_button_cb),
 			   application,
 			   0);
+#else
+  action_group = PANED_NOTEBOOK(application->contents_window)->action_group;
+  gtk_action_group_add_actions(action_group,
+			       toolbar_entries,
+			       G_N_ELEMENTS(toolbar_entries),
+			       application);
+  gtk_action_group_add_toggle_actions(action_group,
+				      toolbar_toggle_entries,
+				      G_N_ELEMENTS(toolbar_toggle_entries),
+				      application);
 
-#if 0
+  ui_manager = PANED_NOTEBOOK(application->contents_window)->ui_manager;
+
+  error = NULL;
+  if (!gtk_ui_manager_add_ui_from_string(ui_manager,
+					 toolbar_ui, -1, &error))
+    {
+      g_message("building toolbar failed: %s\n", error->message);
+      g_error_free(error);
+      exit(EXIT_FAILURE);
+    }
+
+  filter_button
+    = gtk_ui_manager_get_widget(ui_manager,
+				"/PanedNotebookLeftToolbar/BoardToolItems/Filter");
+  GTK_WIDGET_UNSET_FLAGS(filter_button, GTK_CAN_FOCUS);
+  application->filter_button = GTK_TOGGLE_TOOL_BUTTON(filter_button);
+
+# if 1
   {
-    GtkWidget *tmp_widget;
-    tmp_widget = gtk_toggle_button_new_with_label(GTK_STOCK_SORT_ASCENDING);
-    GTK_WIDGET_UNSET_FLAGS(tmp_widget, GTK_CAN_FOCUS);
-    gtk_button_set_use_stock(GTK_BUTTON(tmp_widget), TRUE);
-    gtk_toolbar_prepend_widget(toolbar, tmp_widget,
-			       _("Sort Threadlist by Weight"),
-			       "sort_threadlist_by_weight");
+    /* XXX: ξ줷Τκ */
+    GtkToolButton *tool_button = GTK_TOOL_BUTTON(application->filter_button);
+    GtkWidget *icon_widget = gtk_tool_button_get_icon_widget(tool_button);
+    GtkWidget *label_widget = gtk_tool_button_get_label_widget(tool_button);
+
+    if (icon_widget == NULL && label_widget == NULL)
+      {
+	label_widget = gtk_label_new(gtk_tool_button_get_label(tool_button));
+	gtk_widget_show(label_widget);
+	gtk_tool_button_set_icon_widget(tool_button, label_widget);
+      }
+
+    gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(tool_button), FALSE);
   }
+# endif
 #endif
 
 #if DEBUG_WIDGET_MOST
@@ -452,6 +544,9 @@ initialize_boardlist(OchushaApplication *application)
   g_signal_connect(G_OBJECT(application->contents_window),
 		   "page_switched", G_CALLBACK(page_switched_cb),
 		   application);
+  g_signal_connect(G_OBJECT(application->contents_window),
+		   "page_double_clicked", G_CALLBACK(page_double_clicked_cb),
+		   application);
 
   application->threadlist_search_window = NULL;
 
@@ -730,6 +825,8 @@ font_selection_dialog_response_cb(GtkWidget *dialog, int response_id,
       break;
     }
 
+  gtk_widget_hide(dialog);
+  gtk_widget_unrealize(dialog);
   gtk_widget_destroy(dialog);
 
   if (tree_view_font_name != NULL)
@@ -1201,13 +1298,19 @@ fix_filter_button_state(OchushaApplication *application)
   if (board == NULL)
     return;
 
+#if GTK_MINOR_VERSION <= 2
   gtk_toggle_button_set_active(application->filter_button,
 		ensure_bulletin_board_info(board, application)->enable_filter);
+#else
+  gtk_toggle_tool_button_set_active(application->filter_button,
+				    ensure_bulletin_board_info(board,
+						application)->enable_filter);
+#endif
 }
 
 
 static void
-filter_toggled_cb(GtkToggleButton *toggle_button,
+filter_toggled_cb(GObject *unused,
 		  OchushaApplication *application)
 {
   PanedNotebook *board_pane = PANED_NOTEBOOK(application->contents_window);
@@ -1219,14 +1322,27 @@ filter_toggled_cb(GtkToggleButton *toggle_button,
     return;
 
   info = ensure_bulletin_board_info(board, application);
-  if (info->enable_filter == gtk_toggle_button_get_active(toggle_button))
+#if GTK_MINOR_VERSION <= 2
+  if (info->enable_filter
+      == gtk_toggle_button_get_active(application->filter_button))
     return;
+#else
+  if (info->enable_filter
+      == gtk_toggle_tool_button_get_active(application->filter_button))
+    return;
+#endif
 
   MAIN_THREAD_EVENT_CALLBACK();
 
   info->enable_filter = !info->enable_filter;
 
-  gtk_toggle_button_set_active(toggle_button, info->enable_filter);
+#if GTK_MINOR_VERSION <= 2
+  gtk_toggle_button_set_active(application->filter_button,
+			       info->enable_filter);
+#else
+  gtk_toggle_tool_button_set_active(application->filter_button,
+				    info->enable_filter);
+#endif
 
   OCHU_THREADS_LEAVE();
   redraw_threadlist_view(application, board);
@@ -1387,6 +1503,8 @@ item_view_closed_cb(PanedNotebook *paned_notebook, OchushaBulletinBoard *board,
     }
 
   finalize_bulletin_board_view(board, item_view);
+  gtk_widget_hide(item_view);
+  gtk_widget_unrealize(item_view);
   delayed_g_object_unref(G_OBJECT(item_view));
 }
 
@@ -1435,8 +1553,13 @@ page_switched_cb(PanedNotebook *paned_notebook,
 				     application->threadlist_pane_style);
       gtk_paned_set_position(GTK_PANED(new_page),
 			     application->threadlist_height);
+#if GTK_MINOR_VERSION <= 2
       gtk_toggle_button_set_active(application->filter_button,
 				   info->enable_filter);
+#else
+      gtk_toggle_tool_button_set_active(application->filter_button,
+					info->enable_filter);
+#endif
       tmp_widget = paned_notebook_get_selector(PANED_NOTEBOOK(new_page));
       tmp_widget = gtk_bin_get_child(GTK_BIN(tmp_widget));
       threadlist_view_set_default_open_in_tab(THREADLIST_VIEW(tmp_widget),
@@ -1447,6 +1570,15 @@ page_switched_cb(PanedNotebook *paned_notebook,
 }
 
 
+static void
+page_double_clicked_cb(PanedNotebook *paned_notebook,
+		       OchushaBulletinBoard *board, GtkWidget *item_view,
+		       OchushaApplication *application)
+{
+  refresh_threadlist_view(application, board);
+}
+
+
 static OchushaBulletinBoard *
 board_new_cb(OchushaBBSTable *table, int bbs_type, const gchar *name,
 	     const char *base_url, OchushaApplication *application)
@@ -1585,7 +1717,7 @@ boardlist_read_board_element_cb(OchushaBBSTable *table,
 static void
 boardlist_write_category_element_cb(OchushaBBSTable *table,
 				    OchushaBoardCategory *category,
-				    gzFile *boardlist_xml,
+				    gzFile boardlist_xml,
 				    OchushaApplication *application)
 {
   BoardCategoryGUIInfo *info = ensure_board_category_info(category);
@@ -1640,7 +1772,7 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
 static void
 boardlist_write_board_element_cb(OchushaBBSTable *table,
 				 OchushaBulletinBoard *board,
-				 gzFile *boardlist_xml,
+				 gzFile boardlist_xml,
 				 OchushaApplication *application)
 {
   BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
@@ -2010,7 +2142,7 @@ setup_boardlist(OchushaApplication *application)
 
 
 static void
-make_new_thread_button_cb(GtkWidget *widget, OchushaApplication *application)
+make_new_thread_button_cb(GObject *unused, OchushaApplication *application)
 {
   MAIN_THREAD_EVENT_CALLBACK();
 
@@ -2021,7 +2153,7 @@ make_new_thread_button_cb(GtkWidget *widget, OchushaApplication *application)
 
 
 static void
-open_board_properties_button_cb(GtkWidget *widget,
+open_board_properties_button_cb(GObject *unused,
 				OchushaApplication *application)
 {
   MAIN_THREAD_EVENT_CALLBACK();
@@ -2033,7 +2165,7 @@ open_board_properties_button_cb(GtkWidget *widget,
 
 
 static void
-refresh_board_button_cb(GtkWidget *widget, OchushaApplication *application)
+refresh_board_button_cb(GObject *unused, OchushaApplication *application)
 {
   MAIN_THREAD_EVENT_CALLBACK();
 
@@ -2044,7 +2176,7 @@ refresh_board_button_cb(GtkWidget *widget, OchushaApplication *application)
 
 
 static void
-start_threadlist_search_button_cb(GtkWidget *widget,
+start_threadlist_search_button_cb(GObject *unused,
 				  OchushaApplication *application)
 {
   MAIN_THREAD_EVENT_CALLBACK();
@@ -2181,6 +2313,8 @@ new_thread_dialog_response_cb(GtkWidget *dialog, int response_id,
     {
       if (response_id == GTK_RESPONSE_OK)
 	refresh_threadlist_view(application, board);
+      gtk_widget_hide(dialog);
+      gtk_widget_unrealize(dialog);
       gtk_widget_destroy(dialog);
     }
   else
