@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.33 2004/02/27 22:15:38 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.34 2004/03/21 20:43:48 fuyu Exp $
  */
 
 #include "config.h"
@@ -482,6 +482,7 @@ ochusha_async_buffer_new(char *buffer, size_t length, DestructFunc *destructor)
 
   LOCK_GLOBAL_LOCK;
   sync_object = get_sync_object();
+  buf->sync_object = sync_object;
   ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list, buf);
   UNLOCK_GLOBAL_LOCK;
 
@@ -500,7 +501,6 @@ ochusha_async_buffer_new(char *buffer, size_t length, DestructFunc *destructor)
 
   buf->buffer_length = length;	/* XXX: Ǥ餦٤ġġ*/
 
-  buf->sync_object = sync_object;
   buf->destructor = destructor;
 
 #if DEBUG_MEMORY_USAGE
@@ -522,12 +522,12 @@ ochusha_async_buffer_new_with_file(int fd)
 
   LOCK_GLOBAL_LOCK;
   sync_object = get_sync_object();
+  buf->sync_object = sync_object;
   ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list, buf);
   UNLOCK_GLOBAL_LOCK;
 
   buf->fixed = FALSE;
 
-  buf->sync_object = sync_object;
   buf->destructor = ochusha_async_buffer_free_when_finished;
 
   buf->length = 0;
@@ -600,6 +600,7 @@ ochusha_async_buffer_new_with_file_mmap(int fd)
 
   LOCK_GLOBAL_LOCK;
   sync_object = get_sync_object();
+  buf->sync_object = sync_object;
   ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list, buf);
   UNLOCK_GLOBAL_LOCK;
 
@@ -623,7 +624,6 @@ ochusha_async_buffer_new_with_file_mmap(int fd)
   close(fd);
   
   buf->fixed = FALSE;
-  buf->sync_object = sync_object;
 
 #if DEBUG_MEMORY_USAGE
   fprintf(stderr, "ochusha_async_buffer_new_with_file_mmap(): OchushaAsyncBuffer(%p)\n", buf);
@@ -755,10 +755,23 @@ ochusha_async_buffer_active_unref(OchushaAsyncBuffer *buffer)
 }
 
 
+/*
+ * BUG: Ǥactive_buffer_list˿äƤΤˡ
+ *      GLOBAL_LOCKȤäƤʤ
+ *      ƤӽФƤؿGLOBAL_LOCKȤäƤΤǡñ
+ *      LOCK/UNLOCKǰϤ櫓ˤ⤤ġġ
+ *      GLOBAL_LOCKGLibrecursive mutexѹ٤
+ *      Ȱֻפärecursive mutexcondvarϻȤʤ
+ *      Τ(⤽semanticstrivialˤϷʤʤΤ)
+ *      줫鸫ľɬפäݤ褦嵐
+ *      Ƥ桼active_buffer_listmutexcondvarǻȤ
+ *      mutexƱǤɬפ¿ʬʤΤǡǧƤ顢
+ *      ̤mutexäƲ褷褦
+ */
 void
 ochusha_async_buffer_suspend_all(void)
 {
-  g_slist_foreach(ochusha_async_buffer_list,
+  g_slist_foreach(active_buffer_list,
 		  (GFunc)ochusha_async_buffer_suspend, NULL);
 }
 
@@ -766,7 +779,7 @@ ochusha_async_buffer_suspend_all(void)
 void
 ochusha_async_buffer_resume_all(void)
 {
-  g_slist_foreach(ochusha_async_buffer_list,
+  g_slist_foreach(active_buffer_list,
 		  (GFunc)ochusha_async_buffer_resume, NULL);
 }
 
@@ -774,7 +787,7 @@ ochusha_async_buffer_resume_all(void)
 void
 ochusha_async_buffer_terminate_all(void)
 {
-  g_slist_foreach(ochusha_async_buffer_list,
+  g_slist_foreach(active_buffer_list,
 		  (GFunc)ochusha_async_buffer_terminate, NULL);
 }
 
@@ -782,11 +795,18 @@ ochusha_async_buffer_terminate_all(void)
 void
 ochusha_async_buffer_suspend(OchushaAsyncBuffer *buffer)
 {
+  gboolean dead_buffer;
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr, "suspend threads refering OchushaAsyncBuffer(%p)\n",
 	  buffer);
 #endif
 
+  LOCK_GLOBAL_LOCK;
+  dead_buffer = g_slist_find(active_buffer_list, buffer) == NULL;
+  UNLOCK_GLOBAL_LOCK;
+  if (dead_buffer)
+    return;
+
   LOCK_BUFFER_LOCK(buffer);
   buffer->number_of_suspended_users = 0;
   buffer->state = OCHUSHA_ASYNC_BUFFER_SUSPENDED;
@@ -834,6 +854,14 @@ ochusha_async_buffer_suspend(OchushaAsyncBuffer *buffer)
 void
 ochusha_async_buffer_resume(OchushaAsyncBuffer *buffer)
 {
+  gboolean dead_buffer;
+
+  LOCK_GLOBAL_LOCK;
+  dead_buffer = g_slist_find(active_buffer_list, buffer) == NULL;
+  UNLOCK_GLOBAL_LOCK;
+  if (dead_buffer)
+    return;
+
   LOCK_BUFFER_LOCK(buffer);
   buffer->state = OCHUSHA_ASYNC_BUFFER_OK;
   BROADCAST_BUFFER_CONDITION(buffer);
@@ -844,11 +872,18 @@ ochusha_async_buffer_resume(OchushaAsyncBuffer *buffer)
 void
 ochusha_async_buffer_terminate(OchushaAsyncBuffer *buffer)
 {
+  gboolean dead_buffer;
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr,
 	  "ochusha_async_buffer_terminate: buffer=%p\n", buffer);
 #endif
 
+  LOCK_GLOBAL_LOCK;
+  dead_buffer = g_slist_find(active_buffer_list, buffer) == NULL;
+  UNLOCK_GLOBAL_LOCK;
+  if (dead_buffer)
+    return;
+
   LOCK_BUFFER_LOCK(buffer);
   buffer->state = OCHUSHA_ASYNC_BUFFER_TERMINATED;
   BROADCAST_BUFFER_CONDITION(buffer);
