@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_jbbs.c,v 1.37 2004/02/27 22:17:12 fuyu Exp $
+ * $Id: ochusha_thread_jbbs.c,v 1.38.2.1 2004/05/01 15:45:34 fuyu Exp $
  */
 
 #include "config.h"
@@ -53,6 +53,11 @@
 #else
 # include <oniguruma.h>
 #endif
+
+#if (ONIGURUMA_VERSION_MAJOR >= 2) && (ONIGURUMA_VERSION_MINOR > 0)
+# define ONIGURUMA_2_2_API		1
+#endif
+
 #define ENABLE_HTML_TO_DAT_CONVERTER	1
 
 
@@ -1056,7 +1061,11 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
   gboolean dump_html = FALSE;
   regex_t *title_pattern = NULL;
   regex_t *response_pattern = NULL;
+#ifdef ONIGURUMA_2_2_API
+  OnigRegion *match_region = NULL;
+#else
   RegRegion *match_region = NULL;
+#endif
 
   if (title_pattern_string == NULL)
     initialize_default_regexp_patterns();
@@ -1064,27 +1073,42 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
   switch (thread->board->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
+#ifdef ONIGURUMA_2_2_API
+      if (onig_new(&title_pattern,
+		   (char *)title_pattern_string, (char *)title_pattern_end,
+		   ONIG_OPTION_EXTEND, ONIG_ENCODING_EUC_JP,
+		   ONIG_SYNTAX_POSIX_EXTENDED, NULL) != ONIG_NORMAL)
+#else
       if (regex_new(&title_pattern,
 		    (char *)title_pattern_string, (char *)title_pattern_end,
-#ifdef REG_ENCODING_EUC_JP
+# ifdef REG_ENCODING_EUC_JP
 		    REG_OPTION_EXTEND, REG_ENCODING_EUC_JP,
-#else /* old style */
+# else /* old style */
 		    REG_OPTION_EXTEND, REGCODE_EUCJP,
-#endif
+# endif
 		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
+#endif
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
 	}
+#ifdef ONIGURUMA_2_2_API
+      if (onig_new(&response_pattern,
+		   (char *)jbbs_shitaraba_response_pattern_string,
+		   (char *)jbbs_shitaraba_response_pattern_end,
+		   ONIG_OPTION_EXTEND, ONIG_ENCODING_EUC_JP,
+		   ONIG_SYNTAX_POSIX_EXTENDED, NULL) != ONIG_NORMAL)
+#else
       if (regex_new(&response_pattern,
 		    (char *)jbbs_shitaraba_response_pattern_string,
 		    (char *)jbbs_shitaraba_response_pattern_end,
-#ifdef REG_ENCODING_EUC_JP
+# ifdef REG_ENCODING_EUC_JP
 		    REG_OPTION_EXTEND, REG_ENCODING_EUC_JP,
-#else /* old style */
+# else /* old style */
 		    REG_OPTION_EXTEND, REGCODE_EUCJP,
-#endif
+# endif
 		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
+#endif
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
@@ -1097,27 +1121,42 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
       break;
 
     case OCHUSHA_BBS_TYPE_MACHIBBS:
+#ifdef ONIGURUMA_2_2_API
+      if (onig_new(&title_pattern,
+		   (char *)title_pattern_string, (char *)title_pattern_end,
+		   ONIG_OPTION_EXTEND, ONIG_ENCODING_SJIS,
+		   ONIG_SYNTAX_POSIX_EXTENDED, NULL) != ONIG_NORMAL)
+#else
       if (regex_new(&title_pattern,
 		    (char *)title_pattern_string, (char *)title_pattern_end,
-#ifdef REG_ENCODING_SJIS
+# ifdef REG_ENCODING_SJIS
 		    REG_OPTION_EXTEND, REG_ENCODING_SJIS,
-#else /* old style */
+# else /* old style */
 		    REG_OPTION_EXTEND, REGCODE_SJIS,
-#endif
+# endif
 		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
+#endif
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
 	}
+#ifdef ONIGURUMA_2_2_API
+      if (onig_new(&response_pattern,
+		   (char *)machibbs_response_pattern_string,
+		   (char *)machibbs_response_pattern_end,
+		   ONIG_OPTION_EXTEND, ONIG_ENCODING_SJIS,
+		   ONIG_SYNTAX_POSIX_EXTENDED, NULL) != ONIG_NORMAL)
+#else
       if (regex_new(&response_pattern,
 		    (char *)machibbs_response_pattern_string,
 		    (char *)machibbs_response_pattern_end,
-#ifdef REG_ENCODING_SJIS
+# ifdef REG_ENCODING_SJIS
 		    REG_OPTION_EXTEND, REG_ENCODING_SJIS,
-#else /* old style */
+# else /* old style */
 		    REG_OPTION_EXTEND, REGCODE_SJIS,
-#endif
+# endif
 		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
+#endif
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
@@ -1130,27 +1169,42 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
       break;
 
     case OCHUSHA_BBS_TYPE_MITINOKU:
+#ifdef ONIGURUMA_2_2_API
+      if (onig_new(&title_pattern,
+		   (char *)title_pattern_string, (char *)title_pattern_end,
+		   ONIG_OPTION_EXTEND, ONIG_ENCODING_EUC_JP,
+		   ONIG_SYNTAX_POSIX_EXTENDED, NULL) != ONIG_NORMAL)
+#else
       if (regex_new(&title_pattern,
 		    (char *)title_pattern_string, (char *)title_pattern_end,
-#ifdef REG_ENCODING_EUC_JP
+# ifdef REG_ENCODING_EUC_JP
 		    REG_OPTION_EXTEND, REG_ENCODING_EUC_JP,
-#else /* old style */
+# else /* old style */
 		    REG_OPTION_EXTEND, REGCODE_EUCJP,
-#endif
+# endif
 		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
+#endif
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
 	}
+#ifdef ONIGURUMA_2_2_API
+      if (onig_new(&response_pattern,
+		   (char *)mitinoku_response_pattern_string,
+		   (char *)mitinoku_response_pattern_end,
+		   ONIG_OPTION_EXTEND, ONIG_ENCODING_EUC_JP,
+		   ONIG_SYNTAX_POSIX_EXTENDED, NULL) != ONIG_NORMAL)
+#else
       if (regex_new(&response_pattern,
 		    (char *)mitinoku_response_pattern_string,
 		    (char *)mitinoku_response_pattern_end,
-#ifdef REG_ENCODING_EUC_JP
+# ifdef REG_ENCODING_EUC_JP
 		    REG_OPTION_EXTEND, REG_ENCODING_EUC_JP,
-#else /* old style */
+# else /* old style */
 		    REG_OPTION_EXTEND, REGCODE_EUCJP,
-#endif
+# endif
 		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
+#endif
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
@@ -1200,7 +1254,11 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 
   g_assert(title_pattern != NULL);
   g_assert(response_pattern != NULL);
+#ifdef ONIGURUMA_2_2_API
+  match_region = onig_region_new();
+#else
   match_region = regex_region_new();
+#endif
 
   ochusha_async_buffer_lock(html_buffer);
   while (TRUE)
@@ -1240,12 +1298,23 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 	  fprintf(stderr, "html_line: \"%s\"\n", html_line_buffer);
 #endif
 
+#ifdef ONIGURUMA_2_2_API
+	  onig_region_clear(match_region);
+#else
 	  regex_region_clear(match_region);
+#endif
 	  if (response_number == 1 && title == NULL
+#ifdef ONIGURUMA_2_2_API
+	      && onig_search(title_pattern,
+			     html_line_buffer, html_line_buffer_tail,
+			     html_line_buffer, html_line_buffer_tail,
+			     match_region, 0) >= 0)
+#else
 	      && regex_search(title_pattern,
 			      html_line_buffer, html_line_buffer_tail,
 			      html_line_buffer, html_line_buffer_tail,
 			      match_region, 0) >= 0)
+#endif
 	    {
 	      char *title_head = html_line_buffer + match_region->beg[1];
 	      int title_len = match_region->end[1] - match_region->beg[1];
@@ -1258,10 +1327,17 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 	    {
 	      fprintf(stderr, "html_line: \"%s\"\n", html_line_buffer);
 	    }
+#ifdef ONIGURUMA_2_2_API
+	  else if (onig_search(response_pattern,
+			       html_line_buffer, html_line_buffer_tail,
+			       html_line_buffer, html_line_buffer_tail,
+			       match_region, 0) >= 0)
+#else
 	  else if (regex_search(response_pattern,
 				html_line_buffer, html_line_buffer_tail,
 				html_line_buffer, html_line_buffer_tail,
 				match_region, 0) >= 0)
+#endif
 	    {
 	      int res_num;
 	      char *tag_head = html_line_buffer + match_region->beg[2];
@@ -1566,12 +1642,21 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
     }
 
  terminated:
+#ifdef ONIGURUMA_2_2_API
+  if (title_pattern != NULL)
+    onig_free(title_pattern);
+  if (response_pattern != NULL)
+    onig_free(response_pattern);
+  if (match_region != NULL)
+    onig_region_free(match_region, 1);
+#else
   if (title_pattern != NULL)
     regex_free(title_pattern);
   if (response_pattern != NULL)
     regex_free(response_pattern);
   if (match_region != NULL)
     regex_region_free(match_region, 1);
+#endif
   OCHU_OBJECT_UNREF(dat_buffer);
   OCHU_OBJECT_UNREF(args->broker);
 }
@@ -1716,7 +1801,9 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 
   if (cgi_url[0] != '\0')
     html_buffer = ochusha_network_broker_read_from_url(broker, NULL, cgi_url,
-					NULL, mode, FALSE,
+					NULL,
+					OCHUSHA_NETWORK_BROKER_CACHE_IGNORE,
+					FALSE,
 					broker->config->thread_chunksize);
   else
     html_buffer = NULL;
